from flask import Flask
from flask_cors import CORS
from app.main.v1.api import *
from app.main.v2.api import *
from app.eluent_curve.api import *
from app.exceptions import register_error_handlers
from flask_apispec import FlaskApiSpec, doc
from app.mylogger import get_logger

import requests
import json
from datetime import datetime


logger = get_logger('mylogger', log_file='app.log')



def custom_schema_name_resolver(schema):
    return schema.__class__.__name__


def create_app():
    app = Flask(__name__)


    app.config.update({
        'APISPEC_SWAGGER_URL': '/swagger/',
        'APISPEC_SWAGGER_UI_URL': '/swagger-ui/',
        'APISPEC_SCHEMA_NAME_RESOLVER': custom_schema_name_resolver,
    })

    docs = FlaskApiSpec(app)
    docs.register_existing_resources()

    app.register_blueprint(main_bp_1, url_prefix='/api/v1')
    register_main_v1(docs)

    app.register_blueprint(eluent_curve_blue, url_prefix='/api/eluent')
    eluent_curve_api(docs)

    app.register_blueprint(main_bp_2, url_prefix='/api/v2')
    register_main_v2(docs)
    print(docs.spec.to_dict())
    @app.route('/swagger-json/', methods=['GET'])
    def swagger_json():
        try:
            swagger_dict = docs.spec.to_dict()
            print(json.dumps(swagger_dict, indent=2))  # 打印Swagger JSON数据
            return jsonify(swagger_dict)
        except Exception as e:
            logger.error(f"Error generating Swagger JSON: {e}")
            return jsonify({"error": "Failed to generate Swagger JSON"}), 500

    # CORS(app, resources={r"/api/*": {"origins": "http://localhost:3000"}})
    CORS(app, resources={r"/*": {"origins": "*"}})

    # Register error handlers
    register_error_handlers(app,logger)

    # 启用 CORS

    @app.before_request
    def log_request_info():
        request_info = {
            "method": request.method,
            "url": request.url,
            "headers": dict(request.headers),
            "body": request.get_json() if request.is_json else request.data.decode('utf-8')
        }
        # 保存请求信息以便在after_request中使用
        request.environ['request_info'] = request_info

    @app.after_request
    def log_request_and_response_info(response):
        # 只记录状态码不等于200的响应
        if response.status_code != 200:
            separator = '-' * 80
            logger.info(separator)

            # 记录请求信息
            request_info = request.environ.get('request_info')
            if request_info:
                logger.info(f"Request: {request_info['method']} {request_info['url']}")
                logger.debug(f"Headers: {request_info['headers']}")
                logger.debug(f"Body: {request_info['body']}")

            # 记录响应信息
            logger.info(f"Response: {response.status}")
            logger.debug(f"Response Headers: {dict(response.headers)}")

            # 获取响应体并记录
            response_data = response.get_data(as_text=True)
            logger.debug(f"Response Body: {response_data}")

        return response


    return app



from marshmallow import Schema, fields

class curvePoint(Schema):
    time = fields.Str()
    value = fields.Float()

class curveInPut(Schema):
    start_time = fields.Str()
class curveOutPut(Schema):
    message = fields.Str()
    point = fields.Nested(curvePoint,required=True)

class verticalePoint(Schema):
    timeStart = fields.Str()
    timeEnd = fields.Str()
    tube = fields.Float()

class verticalOutPut(Schema):
    message = fields.Str()
    point = fields.Nested(verticalePoint,required=True)

class lineOutPut(Schema):
    message = fields.Str()
    point = fields.List(fields.Nested(curvePoint), required=True)


from flask import Blueprint,jsonify,request
from flask_apispec.views import MethodResource

from flask_apispec import doc, marshal_with, use_kwargs, FlaskApiSpec
from marshmallow import Schema, fields
from flask_restful import Api, Resource

main_bp_2 = Blueprint('main_v2', __name__)
api = Api(main_bp_2)


class UserSchema(Schema):
    username = fields.Str(required=True)
    email = fields.Email(required=True)


@doc(tags=[" main - v2"])
class UserAPI(MethodResource, Resource):
    @marshal_with(UserSchema(many=True))
    def get(self):
        users = [{'id': 1, 'username': 'user1', 'email': 'user1@example.com'}]  # Mock data
       
        return users

    @use_kwargs(UserSchema, location='json')
    @marshal_with(UserSchema(many=True))
    def post(self,**kwargs):
        user_data = kwargs  # 获取传入的 JSON 数据
        print("Received user data:", user_data)
        users = [{'id': 1, 'username': 'user1', 'email': 'user1@example.com'}]  # Mock data
        users.append(user_data)
        return users






def register_main_v2(docs):
    api.add_resource(UserAPI, '/main_v2')
    docs.register(UserAPI, blueprint="main_v2")



from flask import Blueprint
from flask_apispec.views import MethodResource

from flask_apispec import doc, marshal_with, use_kwargs, FlaskApiSpec
from marshmallow import Schema, fields,validate,ValidationError
from flask_restful import Api, Resource

main_bp_1 = Blueprint('main_v1', __name__)
api = Api(main_bp_1)


class UserSchema(Schema):
    id = fields.Str(dump_only=True,missing=1)
    username = fields.Str()
    email = fields.Str(required=True)
    device_id = fields.Str( description="The ID of the device.",validate=validate.OneOf(["1---", "3", "5"]))

class UserOutPut(Schema):
    message = fields.Str(required=True)


@doc(tags=[" main - v1"])
class UserAPI(MethodResource, Resource):
    @marshal_with(UserSchema(many=True))
    def get(self):
        users = [{'id': 1, 'username': 'user1', 'email': 'user1@example.com'}]  # Mock data
        return users

    @use_kwargs(UserSchema, location='json')
    @marshal_with(UserSchema(many=True))
    def post(self, **kwargs):
        users = [
            {"id": 1, "username": "user1", "email": "user1@example.com"},
            {"id": 2, "username": "user2", "email": "user2@example.com"},
        ]
        return users




def register_main_v1(docs):
    api.add_resource(UserAPI, '/main_v1')
    docs.register(UserAPI, blueprint="main_v1")



import unittest
from app import create_app


class MainTestCase(unittest.TestCase):
    def setUp(self):
        self.app = create_app()
        self.client = self.app.test_client()

    def test_index(self):
        response = self.client.get('/main_v1')
        self.assertEqual(response.status_code, 200)
        self.assertIn('Welcome to the API', response.json['messages'])


if __name__ == '__main__':
    unittest.main()



import logging
import os

# 单例模式的logger
_logger = None

def get_logger(name='mylogger', log_file=None):
    global _logger
    if _logger is None:
        # 创建一个logger
        _logger = logging.getLogger(name)
        _logger.setLevel(logging.DEBUG)  # 设置默认日志级别

        # 创建一个控制台处理器，并设置日志级别为DEBUG
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)

        # 创建一个格式化器，并将其添加到处理器
        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        ch.setFormatter(formatter)

        # 将处理器添加到logger
        _logger.addHandler(ch)

        if log_file:
            # 创建一个文件处理器，并设置日志级别为DEBUG
            fh = logging.FileHandler(log_file)
            fh.setLevel(logging.DEBUG)
            fh.setFormatter(formatter)
            _logger.addHandler(fh)

    return _logger



import unittest
from app import create_app


class UsersTestCase(unittest.TestCase):
    def setUp(self):
        self.app = create_app()
        self.client = self.app.test_client()

    def test_get_users(self):
        response = self.client.get('/users/')
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.json['users'], [1,2,3])


if __name__ == '__main__':
    unittest.main()



import datetime
import unittest
import requests
import json
from app import create_app


class TestEluentCurveAPI(unittest.TestCase):

    def setUp(self):
        self.app = create_app()
        self.client = self.app.test_client()
        # self.url = 'http://127.0.0.1:5000/api/eluent/get_curve'  # 根据你的实际API URL进行调整

    def test_get_curve(self):
        print("-------------------")
        response = self.client.get('/api/eluent/get_curve')
        self.assertEqual(response.status_code, 200)

        data = response.get_json()  # 使用 get_json() 方法解析响应数据
        self.assertIn('message', data)
        self.assertIn('point', data)

        eluent_point = data['point']
        self.assertIn('time', eluent_point)
        self.assertIn('value', eluent_point)

        # 验证时间格式
        try:
            datetime.datetime.strptime(eluent_point['time'], "%H:%M:%S")
        except ValueError:
            self.fail("Incorrect time format")

        # 验证 value 是否在 [0, 1] 范围内
        value = float(eluent_point['value'])
        self.assertGreaterEqual(value, 0)
        self.assertLessEqual(value, 100)


if __name__ == '__main__':
    unittest.main()



def create_response(data, status_code=200):
    return {"data": data, "status": "success"}, status_code

def create_error_response(message, status_code=400):
    return {"message": message, "status": "error"}, status_code



from setuptools import setup, find_packages

setup(
    name='chromayograph',
    version='1.0.0',
    packages=find_packages(),
    install_requires=[
        'Flask',  # 根据你的项目需求添加依赖项
        'requests'
    ],
    entry_points={
        'console_scripts': [
            'chromayograph=main:main',  # main.py 的入口函数
        ],
    },
    author='Zhao Xin Yue',
    author_email='zhaoxinyue@pku.edu.cn',
    description='A brief description of your project',
    url='https://github.com/yue000103/Chromatograph.git',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',
    ],
)



from flask import Blueprint
from flask_apispec.views import MethodResource
from flask_apispec import doc, marshal_with, use_kwargs, FlaskApiSpec
from flask_restful import Api, Resource
import random,math
from app.eluent_curve.schemas import *
from datetime import datetime,timedelta
from dateutil import parser  # 需要安装 python-dateutil 库

eluent_curve_blue = Blueprint('eluent_curve_blue', __name__)
api = Api(eluent_curve_blue)

# 设定波形的周期和振幅
period = 60  # 波形的周期
amplitude = 100  # 波形的振幅
# 初始化波形的相位
phase = random.uniform(0, 2*math.pi)
phase_increment = (2 * math.pi) / period
@doc(tags=[" Get Eluent Curve"])
class GetCurveAPI(MethodResource, Resource):
    @use_kwargs(curveInPut, location='json')
    @marshal_with(curveOutPut)
    def post(self,**kwargs):
        global phase
        random_number = math.sin(phase)
        phase +=  (2 * math.pi) / period  # 控制波形移动的速度
        random_number = (random_number + 1) / 2 * 100 # 将值映射到0到1之

        start_time = parser.parse(kwargs['start_time'])
        current_time = datetime.now()
        time_elapsed = current_time - start_time
        time_str = str(timedelta(seconds=int(time_elapsed.total_seconds())))

        point = {
            "time":time_str,
           "value": random_number
        }
        print(point)
        return {"message": "People Refilling tasks successfully",
             "point":point
              }
current_id = 1
@doc(tags=[" Get Eluent Vertical"])
class GetVerticalAPI(MethodResource, Resource):
    @use_kwargs(curveInPut, location='json')
    @marshal_with(verticalOutPut)
    def post(self,**kwargs):
        global current_id
        current_time = datetime.now()
        start_time = parser.parse(kwargs['start_time'])
        time_elapsed = current_time - start_time
        timeStart = time_elapsed - timedelta(seconds=10)
        timeEnd = time_elapsed - timedelta(seconds=6)
        current_id += 1
        point = {
            "timeStart":timeStart,
            "timeEnd":timeEnd,
            "tube":current_id

        }
        return {"message": "People Refilling tasks","point":point}


@doc(tags=[" Get Eluent Line"])
class GetLinelAPI(MethodResource, Resource):

    @marshal_with(lineOutPut)
    def get(self,):
        point = []
        current_id_line = 1

        time = timedelta(seconds=0)
        for i in range(4):
            time = time + timedelta(seconds=30)
            current_id_line += 10
            value = current_id_line
            formatted_time = (datetime.min + time).strftime('%H:%M:%S')

            point.append({"time":formatted_time,"value":value})
        for i in range(4):
            time = time + timedelta(seconds=30)
            value = current_id_line
            formatted_time = (datetime.min + time).strftime('%H:%M:%S')
            point.append({"time": formatted_time, "value": value})
        print("point",point)
        return {"message": "People Refilling tasks","point":point}



def eluent_curve_api(docs):
    api.add_resource(GetCurveAPI, '/get_curve')
    docs.register(GetCurveAPI, blueprint="eluent_curve_blue")
    api.add_resource(GetVerticalAPI, '/get_vertical')
    docs.register(GetVerticalAPI, blueprint="eluent_curve_blue")
    api.add_resource(GetLinelAPI, '/get_line')
    docs.register(GetLinelAPI, blueprint="eluent_curve_blue")



from flask import jsonify,request


class CustomError(Exception):
    status_code = 400
    def __init__(self, message, status_code=None, payload=None):
        print("__init__")
        Exception.__init__(self)
        self.message = message
        if status_code is not None:
            self.status_code = status_code
        self.payload = payload

    def to_dict(self):
        print("to_dict")

        rv = dict(self.payload or ())
        rv['message'] = self.message
        return rv


def handle_custom_error(error,logger):
    logger.error(f"CustomError: {error.message}")
    logger.error(f"Request: {request.method} {request.url}")
    logger.error(f"Headers: {request.headers}")
    if request.is_json:
        logger.error(f"Body: {request.get_json()}")
    else:
        logger.error(f"Body: {request.data}")

    if error.status_code == 500:
        response = jsonify({'message': '格式不对'})
        response.status_code = 500
    else:
        response = jsonify(error.to_dict())
        response.status_code = error.status_code
    return response
# 自定义 TypeError 的错误处理器
def handle_type_error(error,logger):
    logger.error(f"TypeError: {error}")
    logger.error(f"Request: {request.method} {request.url}")
    logger.error(f"Headers: {request.headers}")
    if request.is_json:
        logger.error(f"Body: {request.get_json()}")
    else:
        logger.error(f"Body: {request.data}")

    response = jsonify({'message': '发生了意外错误: ' + str(error)})
    response.status_code = 500
    return response
# 422  <!doctype html>
# <html lang=en>
# <title>422 Unprocessable Entity</title>
# <h1>Unprocessable Entity</h1>
# <p>The request was well-formed but was unable to be followed due to semantic errors.</p>
def register_error_handlers(app, logger):
    app.register_error_handler(CustomError, lambda error: handle_custom_error(error, logger))
    app.register_error_handler(TypeError, lambda error: handle_type_error(error, logger))
    app.register_error_handler(AttributeError, lambda error: handle_type_error(error, logger))

